==> embrace-android-sdk-4.6.0-sources.jar: io/embrace/android/embracesdk/EmbraceNetworkConnectivityService.java <==
package io.embrace.android.embracesdk;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;

import com.fernandocejas.arrow.checks.Preconditions;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.NavigableMap;
import java.util.TreeMap;

class EmbraceNetworkConnectivityService extends BroadcastReceiver implements NetworkConnectivityService, MemoryCleanerListener {
    private final IntentFilter intentFilter = new IntentFilter(ConnectivityManager.CONNECTIVITY_ACTION);
    private final NavigableMap<Long, NetworkStatus> networkReachable;
    private final ConnectivityManager connectivityManager;
    private final Context context;
    private final Clock clock;

    public EmbraceNetworkConnectivityService(Context context, Clock clock, MemoryCleanerService memoryCleanerService) {
        this.context = Preconditions.checkNotNull(context, "context must not be null");
        this.clock = Preconditions.checkNotNull(clock);
        this.connectivityManager =
                (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
        this.networkReachable = new TreeMap<>();
        Preconditions.checkNotNull(memoryCleanerService).addListener(this);
        try {
            context.registerReceiver(this, intentFilter);
        } catch (Exception ex) {
            EmbraceLogger.logDebug("Failed to register EmbraceNetworkConnectivityService broadcast receiver. Connectivity status will be unavailable.", ex);
        }
    }

    @Override
    public void onReceive(Context context, Intent intent) {
        handleNetworkStatus();
    }

    @Override
    public List<Interval> getNetworkInterfaceIntervals(long startTime, long endTime) {
        synchronized (this) {
            List<Interval> results = new ArrayList<>();
            for (Map.Entry<Long, NetworkStatus> entry : networkReachable.subMap(startTime, endTime).entrySet()) {
                long currentTime = entry.getKey();
                Long next = networkReachable.higherKey(currentTime);
                results.add(new Interval(currentTime, next != null ? next : endTime, entry.getValue().getName()));
            }
            return results;
        }
    }

    @Override
    public void networkStatusOnSessionStarted(long startTime) {
        handleNetworkStatus(startTime);
    }

    private void handleNetworkStatus() {
        handleNetworkStatus(clock.now());
    }

    private void handleNetworkStatus(long timestamp) {
        try {
            NetworkInfo networkInfo = connectivityManager.getActiveNetworkInfo();
            if (networkInfo != null && networkInfo.isConnected()) {
                // Network is reachable
                if (networkInfo.getType() == ConnectivityManager.TYPE_WIFI) {
                    saveStatus(timestamp, NetworkStatus.WIFI);
                } else if (networkInfo.getType() == ConnectivityManager.TYPE_MOBILE) {
                    saveStatus(timestamp, NetworkStatus.WAN);
                }
            } else {
                // Network is not reachable
                saveStatus(timestamp, NetworkStatus.NOT_REACHABLE);
            }
        } catch (Exception ex) {
            EmbraceLogger.logDebug("Failed to record network connectivity", ex);
        }
    }

    private void saveStatus(long timestamp, NetworkStatus networkStatus) {
        synchronized (this) {
            if (networkReachable.isEmpty() || !networkReachable.lastEntry().getValue().equals(networkStatus)) {
                networkReachable.put(timestamp, networkStatus);
            }
        }
    }

    @Override
    public void close() {
        context.unregisterReceiver(this);
    }

    @Override
    public void cleanCollections() {
        this.networkReachable.clear();
    }

    private enum NetworkStatus {
        NOT_REACHABLE("none"),
        WIFI("wifi"),
        WAN("wan");

        private final String name;

        NetworkStatus(String name) {
            this.name = name;
        }

        String getName() {
            return this.name;
        }
    }

}

==> embrace-android-sdk-4.6.6-sources.jar: io/embrace/android/embracesdk/EmbraceNetworkConnectivityService.java <==
package io.embrace.android.embracesdk;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;

import com.fernandocejas.arrow.checks.Preconditions;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.NavigableMap;
import java.util.TreeMap;

class EmbraceNetworkConnectivityService extends BroadcastReceiver implements NetworkConnectivityService, MemoryCleanerListener {

    private final IntentFilter intentFilter = new IntentFilter(ConnectivityManager.CONNECTIVITY_ACTION);
    private final NavigableMap<Long, NetworkStatus> networkReachable = new TreeMap<>();
    private final Context context;
    private final Clock clock;
    private final BackgroundWorker connectivityManagerWorker = BackgroundWorker.ofSingleThread("Connectivity Manager Worker");

    private ConnectivityManager connectivityManager;

    public EmbraceNetworkConnectivityService(Context context, Clock clock, MemoryCleanerService memoryCleanerService) {
        this.context = Preconditions.checkNotNull(context, "context must not be null");
        this.clock = Preconditions.checkNotNull(clock);
        Preconditions.checkNotNull(memoryCleanerService).addListener(this);
        registerConnectivityActionReceiver();
    }

    @Override
    public void onReceive(Context context, Intent intent) {
        handleNetworkStatus();
    }

    @Override
    public List<Interval> getNetworkInterfaceIntervals(long startTime, long endTime) {
        synchronized (this) {
            List<Interval> results = new ArrayList<>();
            for (Map.Entry<Long, NetworkStatus> entry : networkReachable.subMap(startTime, endTime).entrySet()) {
                long currentTime = entry.getKey();
                Long next = networkReachable.higherKey(currentTime);
                results.add(new Interval(currentTime, next != null ? next : endTime, entry.getValue().getName()));
            }
            return results;
        }
    }

    @Override
    public void networkStatusOnSessionStarted(long startTime) {
        handleNetworkStatus(startTime);
    }

    private void handleNetworkStatus() {
        handleNetworkStatus(clock.now());
    }

    private void handleNetworkStatus(long timestamp) {
        try {
            NetworkInfo networkInfo = getConnectivityManager().getActiveNetworkInfo();
            if (networkInfo != null && networkInfo.isConnected()) {
                // Network is reachable
                if (networkInfo.getType() == ConnectivityManager.TYPE_WIFI) {
                    saveStatus(timestamp, NetworkStatus.WIFI);
                } else if (networkInfo.getType() == ConnectivityManager.TYPE_MOBILE) {
                    saveStatus(timestamp, NetworkStatus.WAN);
                }
            } else {
                // Network is not reachable
                saveStatus(timestamp, NetworkStatus.NOT_REACHABLE);
            }
        } catch (Exception ex) {
            EmbraceLogger.logDebug("Failed to record network connectivity", ex);
        }
    }

    private void saveStatus(long timestamp, NetworkStatus networkStatus) {
        synchronized (this) {
            if (networkReachable.isEmpty() || !networkReachable.lastEntry().getValue().equals(networkStatus)) {
                networkReachable.put(timestamp, networkStatus);
            }
        }
    }

    private synchronized ConnectivityManager getConnectivityManager() {
        if (connectivityManager == null) {
            connectivityManager = (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
        }
        return connectivityManager;
    }

    private void registerConnectivityActionReceiver() {
        connectivityManagerWorker.submit(() -> {
            try {
                context.registerReceiver(this, intentFilter);
            } catch (Exception ex) {
                EmbraceLogger.logDebug("Failed to register EmbraceNetworkConnectivityService broadcast receiver. Connectivity status will be unavailable.", ex);
            }
            return null;
        });
    }

    @Override
    public void close() {
        context.unregisterReceiver(this);
    }

    @Override
    public void cleanCollections() {
        this.networkReachable.clear();
    }

    private enum NetworkStatus {
        NOT_REACHABLE("none"),
        WIFI("wifi"),
        WAN("wan");

        private final String name;

        NetworkStatus(String name) {
            this.name = name;
        }

        String getName() {
            return this.name;
        }
    }
}
